
/* Table: DataBin, Owner SYSDBA */
CREATE TABLE DATABIN
(
  IDENT        SMALLINT NOT NULL,
  NAME         CHAR(10) NOT NULL,
  DESCRIPTION  VARCHAR(32000),
    CONSTRAINT DATABIN_IDENT_IS_PKEY PRIMARY KEY ( IDENT ),
    CONSTRAINT DATABIN_NAME_IS_UNIQUE UNIQUE( NAME ),
    CONSTRAINT DATABIN_NAME_IS_CAPS CHECK( NAME = UPPER( NAME ) )
);
COMMIT;


/* Table: HKType, Owner SYSDBA */
CREATE TABLE HKTYPE
(
  IDENT        SMALLINT NOT NULL,
  NAME         CHAR(10) NOT NULL,
  DESCRIPTION  VARCHAR(32000),
    CONSTRAINT HKTYPE_IDENT_IS_PKEY PRIMARY KEY ( IDENT ),
    CONSTRAINT HKTYPE_NAME_IS_UNIQUE UNIQUE( NAME ),
    CONSTRAINT HKTYPE_NAME_IS_CAPS CHECK( NAME = UPPER( NAME ) )
);
COMMIT;


/* Table: OpMode, Owner SYSDBA */
CREATE TABLE OPMODE
(
  IDENT        SMALLINT NOT NULL,
  NAME         CHAR(15) NOT NULL,
  DESCRIPTION  VARCHAR(32000),
    CONSTRAINT OPMODE_IDENT_IS_PKEY PRIMARY KEY ( IDENT ),
    CONSTRAINT OPMODE_NAME_IS_UNIQUE UNIQUE( NAME ),
    CONSTRAINT OPMODE_NAME_IS_CAPS CHECK( NAME = UPPER( NAME ) )
);
COMMIT;


/* Table: WaveForm, Owner SYSDBA */
CREATE TABLE WAVEFORM
(
  IDENT        SMALLINT NOT NULL,
  NAME         CHAR(10) NOT NULL,
  DESCRIPTION  VARCHAR(32000),
    CONSTRAINT WAVEFORM_IDENT_IS_PKEY PRIMARY KEY ( IDENT ),
    CONSTRAINT WAVEFORM_NAME_IS_UNIQUE UNIQUE( NAME ),
    CONSTRAINT WAVEFORM_NAME_IS_CAPS CHECK( NAME = UPPER( NAME ) )
);
COMMIT;


/* Table: TxAntenna, Owner SYSDBA */
CREATE TABLE TXANTENNA
(
  IDENT        SMALLINT NOT NULL,
  NAME         CHAR(40) NOT NULL,
  DESCRIPTION  VARCHAR(32000),
    CONSTRAINT TXANTENNA_IDENT_IS_PKEY PRIMARY KEY ( IDENT ),
    CONSTRAINT TXANTENNA_NAME_IS_UNIQUE UNIQUE( NAME )
);
COMMIT;


/* Table: FileDesc, Owner SYSDBA */
CREATE TABLE FILEDESC
(
  IDENT        SMALLINT NOT NULL,
  NAME         CHAR(64) NOT NULL,
  FILETIME_M   INTEGER NOT NULL,
  FILETIME_L   INTEGER NOT NULL,
  FILESIZE_M   INTEGER NOT NULL,
  FILESIZE_L   INTEGER NOT NULL,
    CONSTRAINT FILE_IDENT_IS_PKEY PRIMARY KEY ( IDENT ),
    CONSTRAINT FILE_NAME_IS_UNIQUE UNIQUE( NAME )
);
COMMIT;


/* Table: Program, Owner SYSDBA */
CREATE TABLE PROGRAM
(
  IDENT        INTEGER NOT NULL,
  PROGRAM_NO   SMALLINT NOT NULL,	
    /* Lower Frequency Limit in kHz, 3kHz - 3MHz */
  LOWER_FREQ   SMALLINT NOT NULL, 
    /* Coarse Frequency Step */
    /* if > 0 then log step in % (1-100) */
    /* if < 0 then linear step in 100Hz (100Hz-1MHz) */
  COARSE_FREQ_STEP SMALLINT NOT NULL, 
    /* Upper Frequency Limit in kHz, 3kHz - 3MHz */
  UPPER_FREQ   SMALLINT NOT NULL, 
    /* Fine Frequency Step in 100Hz, 100Hz - 1MHz */
  FINE_FREQ_STEP SMALLINT NOT NULL, 
    /* Number of fine steps */
  NO_OF_FINE_FREQ_STEPS SMALLINT NOT NULL, 
    /* Tx Waveform */
  WAVEFORM_ID  SMALLINT NOT NULL,
    /* Phase Switching flag, 1 - on, 0 - off */
  PHASE_SWITCH SMALLINT NOT NULL,
    /* Tx Antenna options */
  ANTENNA_ID   SMALLINT NOT NULL,
    /* Coupler on flag, 1 - on, 0 - off */
  COUPLER_ON   SMALLINT NOT NULL,
    /* Number of Integrated Repetetion - Negative means power integration instead of coheren one */
  NO_OF_REPEAT SMALLINT NOT NULL,
    /* Pulse Repetetion Rate - number of pulses per second, 0,1,2,4,10,20,50 */
  PULSE_REP_RATE  SMALLINT NOT NULL,  /* per second */
    /* Operating Mode */
  OPMODE_ID    SMALLINT NOT NULL,
    /* Power Limit constraint in watts, 0 - 120 */
  POWER_LIMIT  SMALLINT NOT NULL,
    /* Start Range in 960 km units, 0 - 225 */
  START_RANGE  SMALLINT NOT NULL,
    /* Range Step (resolution) in 10 km units, 24 or 48 */
  RANGE_STEP   SMALLINT NOT NULL,
    /* Number of Range Bins, 8 - 1024 */
  NO_OF_RANGE_BINS SMALLINT NOT NULL,
    /* Base Gain */
  BASE_GAIN    SMALLINT NOT NULL,
    /* Frequency Search, 0 = Disabled, n = 5 freq spaced by n*244 Hz */
  FREQ_SEARCH  SMALLINT NOT NULL,
    /* Number of Range stored, 1 - 1024 */
  NO_OF_RANGE_STORED  SMALLINT NOT NULL,
    /* Bottom of Range Window in 10 Mm units, 0 - 250 */
  BOTTOM_RANGE SMALLINT NOT NULL,
    /* Top of Range Window in 10 Mm units, 0 - 250 */
  TOP_RANGE    SMALLINT NOT NULL,
    /* Databin Format */
  DATABIN_ID   SMALLINT NOT NULL,
    /* Threshold cleaning in %, 0 - 99 */
  THRESHOLD_CLEANING  SMALLINT NOT NULL,
    /* Spare program parameter */
  SPARE_PROG_PAR   SMALLINT NOT NULL,
    /* High RF Noise setting, 0 (no) or 1 (yes) */
  HIGH_RF      SMALLINT NOT NULL,
    /* CIT length in ms, 0 - 650sec */
  CIT_LENGTH   SMALLINT NOT NULL,
    /* Data Status Flag, N/A   ???(AL) */
  DATA_STATUS  SMALLINT NOT NULL,
  MAX_RANGRE   COMPUTED BY ( CAST ( 960 * START_RANGE + NO_OF_RANGE_BINS * RANGE_STEP * 10 AS INTEGER ) ),
  DESCRIPTION  VARCHAR(32000),
    CONSTRAINT PROGRAM_IDENT_IS_PKEY PRIMARY KEY( IDENT ),
    CONSTRAINT PROGRAM_DATABIN_ID_IS_FKEY FOREIGN KEY( DATABIN_ID )
               REFERENCES DATABIN ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT PROGRAM_OPMODE_ID_IS_FKEY FOREIGN KEY( OPMODE_ID )
               REFERENCES OPMODE ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT PROGRAM_WAVEFORM_ID_IS_FKEY FOREIGN KEY( WAVEFORM_ID )
               REFERENCES WAVEFORM ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT PROGRAM_ANTENNA_ID_IS_FKEY FOREIGN KEY( ANTENNA_ID )
               REFERENCES TXANTENNA ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT PROGRAM_PROGRAM_NO_GT0_LT65
                 CHECK( PROGRAM_NO > 0 AND PROGRAM_NO < 65 )
);
COMMIT;


/* Table: ClockDrift, Owner SYSDBA */
CREATE TABLE CLOCKDRIFT
(
  MET_IN_SEC   DOUBLE PRECISION NOT NULL,
  GMT          TIMESTAMP NOT NULL,
    CONSTRAINT CLOCKDRIFT_MET_IS_PKEY PRIMARY KEY ( MET_IN_SEC ),
    CONSTRAINT CLOCKDRIFT_GMT_IS_UNIQUE UNIQUE( GMT ),
    CONSTRAINT CLOCKDRIFT_MET_IS_POSITIVE CHECK( MET_IN_SEC >= 0 )
);
COMMIT;


/* Table: SessionHead, Owner SYSDBA */
CREATE TABLE SessionHead
(
  Ident        INTEGER NOT NULL,
  MET_In_Sec   DOUBLE PRECISION NOT NULL,
  Program_ID   INTEGER NOT NULL,
  Schedule_No  SMALLINT NOT NULL,
  File_ID      SMALLINT NOT NULL,
  Offset_M     INTEGER NOT NULL,
  Offset_L     INTEGER NOT NULL,
  Time_UT      TIMESTAMP NOT NULL,
    CONSTRAINT SessionHead_Ident_IS_PKEY PRIMARY KEY( Ident ),
    CONSTRAINT SessionHead_MET_IS_UNIQUE UNIQUE( MET_In_Sec ),
    CONSTRAINT SessionHead_MET_IS_POSITIVE CHECK( MET_In_Sec >= 0 ),
    CONSTRAINT SessionHead_Time_UT_IS_UNIQUE UNIQUE( Time_UT ),
    CONSTRAINT SessionHead_Program_ID_IS_FKEY FOREIGN KEY( Program_ID )
               REFERENCES Program ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT SessionHead_File_ID_IS_FKEY FOREIGN KEY( File_ID )
               REFERENCES FileDesc ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT SessionHead_ScheduleNo_GE0_LT33
                 CHECK( Schedule_No >= 0 AND Schedule_No < 33 ),
    CONSTRAINT SessionHead_Offset_IS_POSITIVE CHECK( Offset_M >= 0 )
);
COMMIT;


/* Table: SessionBlob, Owner SYSDBA */
CREATE TABLE SessionBlob
(
  Session_ID   INTEGER NOT NULL,
  Data         BLOB NOT NULL,
    CONSTRAINT SessionBlob_Ident_IS_PKEY PRIMARY KEY( Session_ID ),
    CONSTRAINT SessionBlob_Ident_IS_FKEY FOREIGN KEY( Session_ID )
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE
);
COMMIT;


/* Table: Expert, Owner SYSDBA */
CREATE TABLE Expert
(
  Ident        SMALLINT NOT NULL,
  LogName      CHAR(10) NOT NULL,
  FName        CHAR(20) NOT NULL,
  LName        CHAR(20) NOT NULL,
  Cyber        SMALLINT DEFAULT 0 NOT NULL,
    CONSTRAINT Expert_Ident_is_Primary_Key PRIMARY KEY( Ident ),
    CONSTRAINT Expert_LogName_is_Unique UNIQUE( LogName ),
    CONSTRAINT Expert_LogName_is_Caps CHECK( UPPER(LogName) = LogName )
);
COMMIT;


/* Table: RateCat, Owner SYSDBA */
CREATE TABLE RateCat
(
  Ident        SMALLINT NOT NULL,	
  ShortName    CHAR(10) NOT NULL,
  FullName     CHAR(20) NOT NULL,
    CONSTRAINT RateCat_Ident_is_Primary_Key PRIMARY KEY( Ident ),
    CONSTRAINT RateCat_ShortName_is_Unique UNIQUE( ShortName ),
    CONSTRAINT RateCat_ShortName_is_Caps CHECK( UPPER(ShortName) = ShortName )
);
COMMIT;

/* Table: AllRates, Owner SYSDBA */
CREATE TABLE AllRates
(
  Expert_ID    SMALLINT NOT NULL,	
  Session_ID   INTEGER NOT NULL,	
  RateCat_ID   SMALLINT NOT NULL,	
  Rate         SMALLINT NOT NULL,
  Published    SMALLINT DEFAULT 1 NOT NULL,
  SubmissionUT TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    CONSTRAINT AllRates_S_Exp_Cat_is_Prim_Key 
      PRIMARY KEY( Session_ID, Expert_ID, RateCat_ID ),
    CONSTRAINT AllRates_Expert_ID_is_FKEY FOREIGN KEY( Expert_ID ) 
               REFERENCES Expert ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT AllRates_Session_ID_is_FKey FOREIGN KEY( Session_ID ) 
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT AllRates_RateCat_ID_is_FKey FOREIGN KEY( RateCat_ID ) 
               REFERENCES RateCat ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT AllRates_Published_is_Logical 
               CHECK( Published = 0 OR Published = 1 ) 
);
COMMIT;


/* Table: AllSnCommt, Owner SYSDBA */
CREATE TABLE AllSnComm
(
  Expert_ID    SMALLINT NOT NULL,
  Session_ID   INTEGER NOT NULL,
  Comment      BLOB NOT NULL,
  Published    SMALLINT DEFAULT 1 NOT NULL,
  SubmissionUT TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    CONSTRAINT AllSnComm_Exp_SID_is_Prim_Key
      PRIMARY KEY( Expert_ID, Session_ID ),
    CONSTRAINT AllSnComm_Expert_ID_is_FKey FOREIGN KEY( Expert_ID )
               REFERENCES Expert ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT AllSnComm_Session_ID_is_FKey FOREIGN KEY( Session_ID )
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT AllSnComm_Published_is_Logical
               CHECK( Published = 0 OR Published = 1 )

);
COMMIT;


/* Table: SnBBVPar, Owner SYSDBA 
Optimal setttings of BinBrowser display
*/
CREATE TABLE SnBBVPar
(
  Expert_ID    SMALLINT NOT NULL,
  Session_ID   INTEGER NOT NULL,
  Params       VARCHAR( 2000 ) NOT NULL,
    CONSTRAINT SnBBVPar_Exp_SID_is_Prim_Key
               PRIMARY KEY( Expert_ID, Session_ID ),
    CONSTRAINT SnBBVPar_ExpertT_ID_is_FKey FOREIGN KEY( Expert_ID )
               REFERENCES Expert ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT SnBBVPar_Session_ID_is_FKey FOREIGN KEY( Session_ID )
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE
);
COMMIT;


/* Table: TrParCat, Owner SYSDBA 
Categories of parameters for each trace
*/
CREATE TABLE TrParCat
(
  Ident        SMALLINT NOT NULL,	
  ShortName    CHAR(10) NOT NULL,
  FullName     CHAR(50) NOT NULL,
    CONSTRAINT TrParCat_Ident_is_Primary_Key PRIMARY KEY( Ident ),
    CONSTRAINT TrParCat_ShortName_is_Unique UNIQUE( ShortName ),
    CONSTRAINT TrParCat_ShortName_is_Caps CHECK( UPPER(ShortName) = ShortName )
);
COMMIT;


/* Table: TraceCat, Owner SYSDBA 
Categories (arrays) for each trace
*/
CREATE TABLE TraceCat
(
  Ident        SMALLINT NOT NULL,	
  ShortName    CHAR(10) NOT NULL,
  FullName     CHAR(50) NOT NULL,
    CONSTRAINT TraceCat_Ident_is_Primary_Key PRIMARY KEY( Ident ),
    CONSTRAINT TraceCat_ShortName_is_Unique UNIQUE( ShortName ),
    CONSTRAINT TraceCat_ShortName_is_Caps CHECK( UPPER(ShortName) = ShortName )
);
COMMIT;


/* Table: Trace_ID, Owner SYSDBA */
CREATE TABLE Trace_ID
(
  Ident        INTEGER NOT NULL,
  Expert_ID    SMALLINT NOT NULL,
  Session_ID   INTEGER NOT NULL,
  Trace_No     SMALLINT NOT NULL,
    CONSTRAINT Trace_ID_Ident_is_Primary_Key PRIMARY KEY( Ident ),
    CONSTRAINT Trace_ID_ExID_SID_TrNo_is_Uniq
               UNIQUE( Expert_ID, Session_ID, Trace_NO ),
    CONSTRAINT Trace_ID_Expert_ID_is_FKey FOREIGN KEY( Expert_ID )
               REFERENCES Expert ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT Trace_ID_Session_ID_is_FKey FOREIGN KEY( Session_ID )
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT Trace_ID_Trace_No_is_gt0 CHECK( Trace_No > 0 )
);
COMMIT;


/* Table: TraceData, Owner SYSDBA */
CREATE TABLE TraceData
(
  Trace_ID     INTEGER NOT NULL,	
  TraceCat_ID  SMALLINT NOT NULL,	
  Data         VARCHAR(10000) NOT NULL,
    CONSTRAINT TraceData_TrID_TrCat_ID_is_Uniq
               UNIQUE( Trace_ID, TraceCat_ID ),
    CONSTRAINT TraceData_Trace_ID_is_FKey FOREIGN KEY( Trace_ID ) 
               REFERENCES Trace_ID ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT TraceData_TraceCat_ID_is_FKey FOREIGN KEY( TraceCat_ID ) 
               REFERENCES TraceCat ON DELETE CASCADE ON UPDATE CASCADE
);
COMMIT;


/* Table: TracePar, Owner SYSDBA */
CREATE TABLE TracePar
(
  Trace_ID     INTEGER NOT NULL,	
  TrParCat_ID  SMALLINT NOT NULL,	
  Param        DOUBLE PRECISION NOT NULL,
  SubmissionUT TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    CONSTRAINT TrPar_Tr_ID_TrParCat_ID_is_Uniq
               UNIQUE( Trace_ID, TrParCat_ID ),
    CONSTRAINT TrPar_Trace_ID_is_FKey FOREIGN KEY( Trace_ID ) 
               REFERENCES Trace_ID ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT TrPar_TrParCat_ID_is_FKey FOREIGN KEY( TrParCat_ID ) 
               REFERENCES TrParCat ON DELETE CASCADE ON UPDATE CASCADE
);
COMMIT;


/* Table: TrComment, Owner SYSDBA */
CREATE TABLE TrComment
(
  Trace_ID     INTEGER NOT NULL,	
  Comment      BLOB NOT NULL,	
    CONSTRAINT TrComment_Trace_ID_is_FKey FOREIGN KEY( Trace_ID ) 
               REFERENCES Trace_ID ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT TrComment_Trace_ID_is_Unique UNIQUE( Trace_ID ) 
);
COMMIT;


/* Table: ResParCat, Owner SYSDBA 
Categories of parameters for each Resonance Set
*/
CREATE TABLE ResParCat
(
  Ident        SMALLINT NOT NULL,	
  ShortName    CHAR(10) NOT NULL,
  FullName     CHAR(50) NOT NULL,
    CONSTRAINT ResParCat_Ident_is_Primary_Key PRIMARY KEY( Ident ),
    CONSTRAINT ResParCat_ShortName_is_Unique UNIQUE( ShortName ),
    CONSTRAINT ResParCat_ShortName_is_Caps CHECK( UPPER(ShortName) = ShortName )
);
COMMIT;


/* Table: ResPar, Owner SYSDBA
Values of parameters for Resonance Set
*/
CREATE TABLE ResPar
(
  Expert_ID    SMALLINT NOT NULL,
  Session_ID   INTEGER NOT NULL,
  ParCat_ID    SMALLINT NOT NULL,
  Param        DOUBLE PRECISION NOT NULL,
    CONSTRAINT ExID_SnID_ParCatID_is_Uniq
               UNIQUE( Expert_ID, Session_ID, ParCat_ID ),
    CONSTRAINT ResPar_Expert_ID_is_FKey FOREIGN KEY( Expert_ID )
               REFERENCES Expert ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT ResPar_Session_ID_is_FKey FOREIGN KEY( Session_ID )
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT ResPar_ParCat_ID_is_FKey FOREIGN KEY( ParCat_ID )
               REFERENCES ResParCat ON DELETE CASCADE ON UPDATE CASCADE
);
COMMIT;


/* Table: ResCat, Owner SYSDBA 
Categories of additional resonance information in ResData
*/
CREATE TABLE ResCat
(
  Ident        SMALLINT NOT NULL,	
  ShortName    CHAR(10) NOT NULL,
  FullName     CHAR(50) NOT NULL,
    CONSTRAINT ResCat_Ident_is_Primary_Key PRIMARY KEY( Ident ),
    CONSTRAINT ResCat_ShortName_is_Unique UNIQUE( ShortName ),
    CONSTRAINT ResCat_ShortName_is_Caps CHECK( UPPER(ShortName) = ShortName )
);
COMMIT;


/* Table: ResData, Owner SYSDBA
Additional resonance information attached to the measurement
Currently used to hold array of frequencies
For example, array of unmatched frequencies, array of fQn frequencies)
*/
CREATE TABLE ResData
(
  Expert_ID    SMALLINT NOT NULL,
  Session_ID   INTEGER NOT NULL,
  Cat_ID       SMALLINT NOT NULL,
  Data         VARCHAR(4000) NOT NULL,
    CONSTRAINT ExID_SnID_CatID_is_Uniq
               UNIQUE( Expert_ID, Session_ID, Cat_ID ),
    CONSTRAINT ResData_Expert_ID_is_FKey FOREIGN KEY( Expert_ID )
               REFERENCES Expert ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT ResData_Session_ID_is_FKey FOREIGN KEY( Session_ID )
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT ResData_Cat_ID_is_FKey FOREIGN KEY( Cat_ID )
               REFERENCES ResCat ON DELETE CASCADE ON UPDATE CASCADE
);
COMMIT;


/* Table: SPredict, Owner SYSDBA */
CREATE TABLE SPredict
(
  Session_ID   INTEGER NOT NULL,
  DMP          FLOAT NOT NULL, /* Distance to magnetopause in RE */
  DPLA         FLOAT NOT NULL, /* Distance to plasmasphere in RE */
  FCE          FLOAT NOT NULL, /* local electron cyclotron frequency in kilo Hertz */
  FPE          FLOAT NOT NULL, /* local electron plasma frequency in kilo Hertz */
  FMP          FLOAT NOT NULL, /* maximum frequency in kilo Hertz of the echo from the MODEL magnetopause */
  PFMP         FLOAT NOT NULL, /* maximum power flux in Watts per Meter squared from the MODEL magnetopause */
  MLT          FLOAT NOT NULL, /* magnetic local time in hours */
  MLAT         FLOAT NOT NULL, /* magnetic latitude in degrees */
  ILAT         FLOAT NOT NULL, /* ivariant latitude in degrees */
  L            DOUBLE PRECISION NOT NULL, /* dipole lshell value in units of earth radii */
  R            FLOAT NOT NULL, /* radial position in RE */
  XSM          FLOAT NOT NULL, /* x position of spacecraft in RE in solar magnetic coordinates */
  YSM          FLOAT NOT NULL, /* y position of spacecraft in RE in solar magnetic coordinates */
  ZSM          FLOAT NOT NULL, /* z position of spacecraft in RE in solar magnetic coordinates */
  XMP          FLOAT NOT NULL, /* x position of magnetopause in RE in solar magnetic coordinates */
  YMP          FLOAT NOT NULL, /* y position of magnetopause in RE in solar magnetic coordinates */
  ZMP          FLOAT NOT NULL, /* z position of magnetopause in RE in solar magnetic coordinates */
  LATN         FLOAT NOT NULL, /* Geographic!!!!! latitude of north footprint */  
                               /* = 999.0 if not calculable */
  LONN         FLOAT NOT NULL, /* Geographic!!!!! longitude of north footprint */  
                               /* = 999.0 if not calculable */
  DN           FLOAT NOT NULL, /* Distance (in RE) from satellite to north footprint */  
                               /* = -1.0 if not calculable */
  LATS         FLOAT NOT NULL, /* Geographic!!!!! latitude of south footprint */  
                               /* = 999.0 if not calculable */
  LONS         FLOAT NOT NULL, /* Geographic!!!!! longitude of south footprint */  
                               /* = 999.0 if not calculable */
  DS           FLOAT NOT NULL, /* Distance (in RE) from satellite to south footprint */  
                               /* = -1.0 if not calculable */
  XGCI         FLOAT NOT NULL, /* x position of satellite in km in GCI coordinates */
  YGCI         FLOAT NOT NULL, /* y position of satellite in km in GCI coordinates */
  ZGCI         FLOAT NOT NULL, /* z position of satellite in km in GCI coordinates */
  DF           SMALLINT NOT NULL, /* = 1, if orbit parameters are definitive */
  MLON         FLOAT NOT NULL, /* position of satellite in GM (geomagnetic) coordinats - longitude in degrees*/
    CONSTRAINT SPredict_Session_ID_is_Unique UNIQUE( Session_ID ),
    CONSTRAINT SPredict_Session_ID_is_FKey FOREIGN KEY( Session_ID )
               REFERENCES SessionHead ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT SPredict_DPLA_is_ge0 CHECK( DPLA >= 0 ),
    CONSTRAINT SPredict_FCE_is_ge0 CHECK( FCE >= 0 ),
    CONSTRAINT SPredict_FPE_is_ge0 CHECK( FPE >= 0 ),
    CONSTRAINT SPredict_FMP_is_ge0 CHECK( FMP >= 0 ),
    CONSTRAINT SPredict_PFMP_is_ge0 CHECK( PFMP >= 0 ),
    CONSTRAINT SPredict_MLT_is_ge0_and_le24 CHECK( MLT >= 0 AND MLT <= 24 ),
    CONSTRAINT SPredict_MLAT_is_geM90_and_le90 CHECK( MLAT >= -90 AND MLAT <= 90 ),
    CONSTRAINT SPredict_ILAT_is_ge0_and_le90 CHECK( ILAT >= 0 AND ILAT <= 90 ),
    CONSTRAINT SPredict_R_is_gt0 CHECK( R >= 0 ),
    CONSTRAINT SPredict_DF_IS_Restricted CHECK( DF = -1 OR DF = 0 OR DF = 1 ),
    CONSTRAINT Prediction_MLON_IS_GE0_A_LE360 CHECK( MLON >= 0 AND MLON <= 360 )
);
COMMIT;


/* Table: HkHead, Owner SYSDBA */
CREATE TABLE HkHead
(
  IDENT        INTEGER NOT NULL,
  MET_IN_SEC   DOUBLE PRECISION NOT NULL,
  FILE_ID      SMALLINT NOT NULL,
  OFFSET_M     INTEGER NOT NULL,
  OFFSET_L     INTEGER NOT NULL,
  TIME_UT      TIMESTAMP NOT NULL,
  AP_ID        SMALLINT NOT NULL,
  SOFTVERSION  SMALLINT NOT NULL,
  CIDPMET_M    INTEGER NOT NULL,
  CIDPMET_L    INTEGER NOT NULL,
  ORBITPHASE   INTEGER NOT NULL,
    CONSTRAINT HKHEAD_IDENT_IS_PKEY PRIMARY KEY( IDENT ),
    CONSTRAINT HKHEAD_MET_IS_POSITIVE CHECK( MET_IN_SEC >= 0 ),
    CONSTRAINT HKHEAD_FILE_ID_IS_FKEY FOREIGN KEY( FILE_ID )
               REFERENCES FILEDESC ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT HKHEAD_AP_ID_IS_FKEY FOREIGN KEY( AP_ID )
               REFERENCES HKTYPE ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT HKHEAD_OFFSET_IS_POSITIVE CHECK( OFFSET_M >= 0 ),
    CONSTRAINT HKHEAD_CIDPMET_IS_POSITIVE CHECK( CIDPMET_M >= 0 )
);
COMMIT;


/* Table: HkBlob, Owner SYSDBA */
CREATE TABLE HkBlob
(
  Hk_ID        INTEGER NOT NULL,
  Data         BLOB NOT NULL,
    CONSTRAINT HkBlob_Ident_IS_PKEY PRIMARY KEY( Hk_ID ),
    CONSTRAINT HkBlob_Ident_IS_FKEY FOREIGN KEY( Hk_ID )
               REFERENCES HkHead ON DELETE CASCADE ON UPDATE CASCADE
);
COMMIT;


/* Table: HPredict, Owner SYSDBA */
CREATE TABLE HPREDICT
(
  SESSION_ID   INTEGER NOT NULL,	
  DMP          FLOAT NOT NULL, /* Distance to magnetopause in RE */
  DPLA         FLOAT NOT NULL, /* Distance to plasmasphere in RE */
  FCE          FLOAT NOT NULL, /* local electron cyclotron frequency in kilo Hertz */
  FPE          FLOAT NOT NULL, /* local electron plasma frequency in kilo Hertz */
  FMP          FLOAT NOT NULL, /* maximum frequency in kilo Hertz of the echo from the MODEL magnetopause */
  PFMP         FLOAT NOT NULL, /* maximum power flux in Watts per Meter squared from the MODEL magnetopause */
  MLT          FLOAT NOT NULL, /* magnetic local time in hours */ 
  MLAT         FLOAT NOT NULL, /* magnetic latitude in degrees */ 
  ILAT         FLOAT NOT NULL, /* ivariant latitude in degrees */ 
  L            DOUBLE PRECISION NOT NULL, /* dipole lshell value in units of earth radii */ 
  R            FLOAT NOT NULL, /* radial position in RE */
  XSM          FLOAT NOT NULL, /* x position of spacecraft in RE in solar magnetic coordinates */
  YSM          FLOAT NOT NULL, /* y position of spacecraft in RE in solar magnetic coordinates */
  ZSM          FLOAT NOT NULL, /* z position of spacecraft in RE in solar magnetic coordinates */
  XMP          FLOAT NOT NULL, /* x position of magnetopause in RE in solar magnetic coordinates */
  YMP          FLOAT NOT NULL, /* y position of magnetopause in RE in solar magnetic coordinates */
  ZMP          FLOAT NOT NULL, /* z position of magnetopause in RE in solar magnetic coordinates */
  LATN         FLOAT NOT NULL, /* Magnetic latitude of north footprint */  
  LONN         FLOAT NOT NULL, /* Magnetic longitude of north footprint */  
  DN           FLOAT NOT NULL, /* Distance (in RE) from satellite to north footprint */  
  LATS         FLOAT NOT NULL, /* Magnetic latitude of south footprint */  
  LONS         FLOAT NOT NULL, /* Magnetic longitude of south footprint */  
  DS           FLOAT NOT NULL, /* Distance (in RE) from satellite to south footprint */  
  XGCI         FLOAT NOT NULL, /* x position of satellite in km in GCI coordinates */
  YGCI         FLOAT NOT NULL, /* y position of satellite in km in GCI coordinates */
  ZGCI         FLOAT NOT NULL, /* z position of satellite in km in GCI coordinates */
  DF           SMALLINT NOT NULL, /* = 1, if orbit parameters are definitive */
    CONSTRAINT HPREDICT_SESSION_ID_IS_UNIQUE UNIQUE( SESSION_ID ),
    CONSTRAINT HPREDICT_SESSION_ID_IS_FKEY FOREIGN KEY( SESSION_ID ) 
               REFERENCES HkHead ON DELETE CASCADE ON UPDATE CASCADE,
    CONSTRAINT HPREDICT_DPLA_IS_GE0 CHECK( DPLA >= 0 ),
    CONSTRAINT HPREDICT_FCE_IS_GE0 CHECK( FCE >= 0 ),
    CONSTRAINT HPREDICT_FPE_IS_GE0 CHECK( FPE >= 0 ),
    CONSTRAINT HPREDICT_FMP_IS_GE0 CHECK( FMP >= 0 ),
    CONSTRAINT HPREDICT_PFMP_IS_GE0 CHECK( PFMP >= 0 ),
    CONSTRAINT HPREDICT_MLT_IS_GE0_AND_LE24 CHECK( MLT >= 0 AND MLT <= 24 ),
    CONSTRAINT HPREDICT_MLAT_IS_GEM90_AND_LE90 CHECK( MLAT >= -90 AND MLAT <= 90 ),
    CONSTRAINT HPREDICT_ILAT_IS_GE0_AND_LE90 CHECK( ILAT >= 0 AND ILAT <= 90 ),
    CONSTRAINT HPREDICT_R_IS_GT0 CHECK( R >= 0 ),
    CONSTRAINT HPredict_DF_IS_Restricted CHECK( DF = -1 OR DF = 0 OR DF = 1 )
);
COMMIT;


/* Table: Prediction, Owner SYSDBA */
CREATE TABLE Prediction
(
  Time_UT      TIMESTAMP NOT NULL,
  DMP          FLOAT NOT NULL, /* Distance to magnetopause in RE */
  DPLA         FLOAT NOT NULL, /* Distance to plasmasphere in RE */
  FCE          FLOAT NOT NULL, /* local electron cyclotron frequency in kilo Hertz */
  FPE          FLOAT NOT NULL, /* local electron plasma frequency in kilo Hertz */
  FMP          FLOAT NOT NULL, /* maximum frequency in kilo Hertz of the echo from the MODEL magnetopause */
  PFMP         FLOAT NOT NULL, /* maximum power flux in Watts per Meter squared from the MODEL magnetopause */
  MLT          FLOAT NOT NULL, /* magnetic local time in hours */
  MLAT         FLOAT NOT NULL, /* magnetic latitude in degrees */
  ILAT         FLOAT NOT NULL, /* ivariant latitude in degrees */
  L            DOUBLE PRECISION NOT NULL, /* dipole lshell value in units of earth radii */
  RE           FLOAT NOT NULL, /* radial position in RE */
  XSM          FLOAT NOT NULL, /* x position of spacecraft in RE in solar magnetic coordinates */
  YSM          FLOAT NOT NULL, /* y position of spacecraft in RE in solar magnetic coordinates */
  ZSM          FLOAT NOT NULL, /* z position of spacecraft in RE in solar magnetic coordinates */
  XMP          FLOAT NOT NULL, /* x position of magnetopause in RE in solar magnetic coordinates */
  YMP          FLOAT NOT NULL, /* y position of magnetopause in RE in solar magnetic coordinates */
  ZMP          FLOAT NOT NULL, /* z position of magnetopause in RE in solar magnetic coordinates */
  LON          FLOAT NOT NULL, /* Longitude */
  LAT          FLOAT NOT NULL, /* Latitude */  
  LATN         FLOAT NOT NULL, /* Magnetic latitude of north footprint */  
  LONN         FLOAT NOT NULL, /* Magnetic longitude of north footprint */  
  DN           FLOAT NOT NULL, /* Distance (in RE) from satellite to north footprint */  
  LATS         FLOAT NOT NULL, /* Magnetic latitude of south footprint */  
  LONS         FLOAT NOT NULL, /* Magnetic longitude of south footprint */  
  DS           FLOAT NOT NULL, /* Distance (in RE) from satellite to south footprint */  
  XGCI         FLOAT NOT NULL, /* x position of satellite in km in GCI coordinates */
  YGCI         FLOAT NOT NULL, /* y position of satellite in km in GCI coordinates */
  ZGCI         FLOAT NOT NULL, /* z position of satellite in km in GCI coordinates */
  DF           SMALLINT NOT NULL, /* = 1, if orbit parameters are definitive */
    CONSTRAINT Prediction_Time_UT_IS_PK PRIMARY KEY( Time_UT ),
    CONSTRAINT Prediction_DPLA_IS_GE0 CHECK( DPLA >= 0 ),
    CONSTRAINT Prediction_FCE_IS_GE0 CHECK( FCE >= 0 ),
    CONSTRAINT Prediction_FPE_IS_GE0 CHECK( FPE >= 0 ),
    CONSTRAINT Prediction_FMP_IS_GE0 CHECK( FMP >= 0 ),
    CONSTRAINT Prediction_PFMP_IS_GE0 CHECK( PFMP >= 0 ),
    CONSTRAINT Prediction_MLT_IS_GE0_AND_LE24 CHECK( MLT >= 0 AND MLT <= 24 ),
    CONSTRAINT Prediction_MLAT_IS_GEM90_A_LE90 CHECK( MLAT >= -90 AND MLAT <= 90 ),
    CONSTRAINT Prediction_ILAT_IS_GE0_AND_LE90 CHECK( ILAT >= 0 AND ILAT <= 90 ),
    CONSTRAINT Prediction_RE_IS_GT0 CHECK( RE >= 0 ),
    CONSTRAINT Prediction_DF_IS_Restricted CHECK( DF = -1 OR DF = 0 OR DF = 1 )
);
COMMIT;


/* Table: OrbitPar, Owner SYSDBA */
CREATE TABLE ORBITPAR
(
  TIME_UT      TIMESTAMP NOT NULL,
  SM_AXIS      DOUBLE PRECISION NOT NULL,
  ECCENTRICITY DOUBLE PRECISION NOT NULL,
  INCLINATION  DOUBLE PRECISION NOT NULL,
  RAAN         DOUBLE PRECISION NOT NULL,
  ARG_OF_PERIG DOUBLE PRECISION NOT NULL,
  TRUE_ANOMALY DOUBLE PRECISION NOT NULL,
  MEAN_ANOMALY DOUBLE PRECISION NOT NULL,
    CONSTRAINT ORBITPAR_TIME_UT_IS_UNIQUE UNIQUE( TIME_UT ),
    CONSTRAINT ORBITPAR_ECCENTRICITY_GT_0 CHECK( ECCENTRICITY > 0 ),
    CONSTRAINT ORBITPAR_INCLINATION_GE0_LE180 
                 CHECK( INCLINATION >= 0 AND INCLINATION <= 180 ),
    CONSTRAINT ORBITPAR_RAAN_GE0_LT360 
                 CHECK( RAAN >= 0 AND RAAN < 360 ),
    CONSTRAINT ORBITPAR_ARG_OF_PERIG_GE0_LT360 
                 CHECK( ARG_OF_PERIG >= 0 AND ARG_OF_PERIG < 360 ),
    CONSTRAINT ORBITPAR_TRUE_ANOMALY_GE0_LT360 
                 CHECK( TRUE_ANOMALY >= 0 AND TRUE_ANOMALY < 360 ),
    CONSTRAINT ORBITPAR_MEAN_ANOMALY_GE0_LE360 
                 CHECK( MEAN_ANOMALY >= 0 AND MEAN_ANOMALY <= 360 )
);
COMMIT;


/* Table: Pass, Owner SYSDBA */
CREATE TABLE PASS
(
  START_UT     TIMESTAMP NOT NULL, /* Start point time (???) */	
  PERIGEE_UT   TIMESTAMP NOT NULL, /* Perigee passing time */	
  APOGEE_UT    TIMESTAMP NOT NULL, /* Apogee passing time */	
  PERIGEE_KM   DOUBLE PRECISION NOT NULL, /* Distnace Perigee passing time */	
  APOGEE_KM    DOUBLE PRECISION NOT NULL, /* Apogee passing time */	
  PERIOD       DOUBLE PRECISION NOT NULL /* Rotation period in sec */,
    CONSTRAINT PASS_START_UT_IS_UNIQUE UNIQUE( START_UT ),
    CONSTRAINT PASS_PERIGEE_UT_IS_UNIQUE UNIQUE( PERIGEE_UT ),
    CONSTRAINT PASS_APOGEE_UT_IS_UNIQUE UNIQUE( APOGEE_UT ),
    CONSTRAINT PASS_PERIGEE_KM_IS_GT0 CHECK( PERIGEE_KM > 0 ),
    CONSTRAINT PASS_APOGEE_KM_IS_GTPERIGEE_KM CHECK( APOGEE_KM >= PERIGEE_KM ),
    CONSTRAINT PASS_PERIOD_IS_GT0 CHECK(PERIOD > 0 )
);
COMMIT;


/* Table: PosVel, Owner SYSDBA */
CREATE TABLE POSVEL
(
  TIME_UT      TIMESTAMP NOT NULL, /* Measurement UT time */	
  X_KM         DOUBLE PRECISION NOT NULL, /* X-coordinate (KM) in GCI */	
  Y_KM         DOUBLE PRECISION NOT NULL, /* Y-coordinate (KM) in GCI */	
  Z_KM         DOUBLE PRECISION NOT NULL, /* Z-coordinate (KM) in GCI */	
  VX_KM_SEC    DOUBLE PRECISION NOT NULL, /* X-coord of velocity (KM/SEC) in GCI */	
  VY_KM_SEC    DOUBLE PRECISION NOT NULL, /* Y-coord of velocity (KM/SEC) in GCI */	
  VZ_KM_SEC    DOUBLE PRECISION NOT NULL, /* Z-coord of velocity (KM/SEC) in GCI */	
  R_KM         DOUBLE PRECISION NOT NULL, /* distance to the center of Earth */ 
    CONSTRAINT POSVEL_TIME_UT_IS_UNIQUE UNIQUE( TIME_UT ),
    CONSTRAINT POSVEL_R_KM_IS_GT0 CHECK( R_KM > 6371 )
);
COMMIT;


/* !!!don't delete this table - it's for connection availability checking!!! */
/* Table: UserReg, Owner SYSDBA */
CREATE TABLE USERREG
(
  IDENT          SMALLINT  NOT NULL,
  COUNT_ACT      INTEGER   NOT NULL,
  HOST_ADDR      CHAR(20)  NOT NULL,
  APP_NAME       CHAR(20)  NOT NULL,
  APP_VERSION    CHAR(20)  NOT NULL,
  TIME_UT        TIMESTAMP NOT NULL,
  U_NAME         CHAR(40)  NOT NULL,
  OS_NAME        CHAR(20)  NOT NULL,
  OS_VERSION     CHAR(20)  NOT NULL,
  HOST_NAME      CHAR(50)  NOT NULL,
  Srv_Local_Time TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    CONSTRAINT USERREG_IDENT_IS_PKEY PRIMARY KEY ( IDENT ),
    CONSTRAINT USERREG_H_ADDR_A_NAME_IS_UNIQUE UNIQUE( HOST_ADDR, APP_NAME )
);
COMMIT;


/* Table: Version, Owner SYSDBA */
CREATE TABLE VERSION
(
  APP_NAME       CHAR(20)  NOT NULL,
  APP_VERSION    CHAR(20)  NOT NULL,
  TIME_UT        TIMESTAMP NOT NULL,
  MODLEVEL       INT DEFAULT 1 NOT NULL,
  INCOMPATIBLE   INT DEFAULT 0 NOT NULL,
  DESCRIPTION    VARCHAR(32000)    ,
    CONSTRAINT VERSION_NAME_VERSION_IS_PKEY 
      PRIMARY KEY ( APP_NAME, APP_VERSION ),
    CONSTRAINT VERSION_MODLEVEL_IS_0_TO_10 
      CHECK( MODLEVEL > 0 AND MODLEVEL <= 10 ),
    CONSTRAINT VERSION_INCOMPATIBLE_IS_0_OR_1 
      CHECK( INCOMPATIBLE = 0 OR INCOMPATIBLE = 1 )
);
COMMIT;

/* Create AuxPar table */
CREATE TABLE AuxPar
(
  Name        CHAR(50) NOT NULL,
  Val         CHAR(20) NOT NULL,
    CONSTRAINT AuxPar_Name_IS_UNIQUE UNIQUE( Name )
);
COMMIT;


/* And create view Session */
CREATE VIEW Session (Ident, MET_In_Sec, Program_ID, Schedule_No, File_ID,
                     Offset_M, Offset_L, Time_UT, Data)
  AS SELECT h.Ident, h.MET_In_Sec, h.Program_ID, h.Schedule_No, h.File_ID,
            h.Offset_M, Offset_L, h.Time_UT, b.DATA
       FROM SessionHead h, SessionBlob b
       WHERE h.Ident = b.Session_ID;
COMMIT;


CREATE VIEW Rate 
  AS SELECT Expert_ID, Session_ID, RateCat_ID, Rate, SubmissionUT FROM AllRates 
     WHERE Published = 1;
COMMIT;


CREATE VIEW SNComment 
  AS SELECT Expert_ID, Session_ID, Comment, SubmissionUT FROM AllSnComm 
     WHERE Published = 1;
COMMIT;


/* And create view HkHeader */
CREATE VIEW HkHeader ( Ident, MET_in_Sec, File_ID, Offset_M, Offset_L,
                       Time_UT, Ap_ID, SofrVersion, CIDPMET_M, CIDPMET_L,
                       OrbitPhase, Data )
  AS SELECT h.Ident, h.MET_In_Sec, File_ID, h.Offset_M, Offset_L, h.Time_UT,
            h.Ap_ID, h.SoftVersion, h.CIDPMET_M, h.CIDPMET_L,
            h.OrbitPhase, b.Data
       FROM HkHead h, HkBlob b
       WHERE h.Ident = b.Hk_ID;
COMMIT;

/* Create Procedures */

SET TERM !!;

/*
 * This procedure returns one row consisting of one column named PERCENT
 * of INTEGER type, the percent of rated plasmagrams for given period.
 * Parameters:
 *   st          - Start of time interval, TIMESTAMP
 *   et          - End of time interval, TIMESTAMP
 *   expert_ID   - Ident of expert, Integer
 *                 If -1, then any expert
 *   category_ID - Ident of rate category, INTEGER
 *                 If -1, then any category
 *
 */
CREATE PROCEDURE GetPercentageOfRated( st TIMESTAMP, et TIMESTAMP,
 expert_ID INTEGER, category_ID INTEGER )
 RETURNS ( percent INTEGER )
AS
DECLARE VARIABLE total INTEGER;
BEGIN
  IF( expert_ID >= 0 AND category_ID >= 0 ) THEN
    SELECT COUNT( DISTINCT( S.Time_UT ) )
       FROM SessionHead S
       WHERE EXISTS
              ( SELECT Rate
                       FROM AllRates R
                       WHERE S.Ident = R.Session_ID
                             AND R.Expert_ID = :expert_ID
                             AND R.RateCat_ID = :category_ID
                       PLAN (R INDEX (RDB$PRIMARY22))
                       )
             AND S.Time_UT >= :st AND S.Time_UT < :et
       PLAN SORT ((S INDEX (RDB$9)))
       INTO :percent;

  IF( expert_ID >= 0 AND category_ID < 0 ) THEN
    SELECT COUNT( DISTINCT( S.Time_UT ) )
       FROM SessionHead S
       WHERE EXISTS
              ( SELECT Rate
                       FROM AllRates R
                       WHERE S.Ident = R.Session_ID
                             AND R.Expert_ID = :expert_ID
                       PLAN (R INDEX (RDB$PRIMARY22))
                       )
             AND S.Time_UT >= :st AND S.Time_UT < :et
       PLAN SORT ((S INDEX (RDB$9)))
       INTO :percent;

  IF( expert_ID < 0 AND category_ID >= 0 ) THEN
    SELECT COUNT( DISTINCT( S.Time_UT ) )
       FROM SessionHead S
       WHERE EXISTS
              ( SELECT Rate
                       FROM AllRates R
                       WHERE S.Ident = R.Session_ID
                             AND R.RateCat_ID = :category_ID
                       PLAN (R INDEX (RDB$PRIMARY22))
                       )
             AND S.Time_UT >= :st AND S.Time_UT < :et
       PLAN SORT ((S INDEX (RDB$9)))
       INTO :percent;

  IF( expert_ID < 0 AND category_ID < 0 ) THEN
    SELECT COUNT( DISTINCT( S.Time_UT ) )
       FROM SessionHead S
       WHERE EXISTS
              ( SELECT Rate
                       FROM AllRates R
                       WHERE S.Ident = R.Session_ID
                       PLAN (R INDEX (RDB$PRIMARY22))
                       )
             AND S.Time_UT >= :st AND S.Time_UT < :et
       PLAN SORT ((S INDEX (RDB$9)))
       INTO :percent;

  SELECT COUNT(*) FROM SessionHead S, Program P
                  WHERE S.Program_ID = P.Ident AND P.OPMode_ID = 3
                  AND S.Time_UT >= :st AND S.Time_UT < :et
                  INTO :total;
  IF( total = 0 )
  THEN percent = 0;
  ELSE percent = ( 100 * percent ) / total;

  SUSPEND;

END!!


/*
 * This procedure returns rows consisting of two columns named D of
 * TIMESTAMP type and PERCENT of INTEGER type. D is the date and PERCENT
 * is the percent of rated plasmagrams.
 * Note, rows are delivered by date increasing order.
 * Parameters:
 *   st       - Start of time interval, TIMESTAMP
 *   et       - End of time interval, TIMESTAMP
 *   expert   - Login name of expert, CHAR(10)
 *              If empty, then any expert
 *   category - Name (short name) of rate category, CHAR(10)
 *              If empty, then any category
 *
 */
CREATE PROCEDURE GetPercentageOfRatedByDays( st TIMESTAMP, et TIMESTAMP,
 expert CHAR(10), category CHAR(10) )
 RETURNS ( d TIMESTAMP, percent INTEGER )
AS
DECLARE VARIABLE expert_ID INTEGER;
DECLARE VARIABLE category_ID INTEGER;
DECLARE VARIABLE sd DATE;
DECLARE VARIABLE ed DATE;
BEGIN
  IF( expert = '' ) THEN
    expert_ID = -1;
  ELSE
    SELECT Ident FROM Expert WHERE LogName = :expert INTO :expert_ID;

  IF( category = '' ) THEN
    category_ID = -1;
  ELSE
    SELECT Ident FROM RateCat WHERE ShortName = :category INTO :category_ID;
  sd = CAST( st AS DATE );
  ed = CAST( et AS DATE );
  WHILE( sd <= ed )
  DO
  BEGIN
    d = sd;
    st = CAST( sd AS TIMESTAMP );
    et = CAST( (sd+1) AS TIMESTAMP );
    SELECT Percent 
      FROM GetPercentageOfRated(:st, :et, :expert_ID, :category_ID)
      INTO :percent;
    SUSPEND;
    sd = sd + 1;
  END
END!!

SET TERM ;!!


/* Create indices */

/* for table Program */
CREATE INDEX PROGRAM_PROGRAM_NO ON PROGRAM( PROGRAM_NO );
CREATE INDEX PROGRAM_DATABIN ON PROGRAM( DATABIN_ID );

/* for table OrbiPar */
CREATE DESCENDING INDEX DescIndOnOrbitParForTime ON ORBITPAR (TIME_UT);

/* for table Pass */
CREATE DESCENDING INDEX DescIndOnPassForPerigee_UT ON PASS (PERIGEE_UT);
CREATE DESCENDING INDEX DescIndOnPassForApogee_UT  ON PASS (APOGEE_UT);

/* for table SessionHead */
CREATE DESCENDING INDEX DescIndOnSessionForMET_IN_SEC ON SessionHead (MET_in_Sec);
CREATE DESCENDING INDEX DescIndOnSessionForTime_UT ON SessionHead (Time_UT);

/* for table HKHead */
CREATE DESCENDING INDEX DescIndOnHKHeaderForMET_IN_SEC ON HkHead (MET_in_Sec);
CREATE ASCENDING  INDEX AscIndOnHKHeaderForTime_UT  ON HkHead (Time_UT);
CREATE DESCENDING INDEX DescIndOnHKHeaderForTime_UT ON HkHead (Time_UT);
CREATE DESCENDING INDEX DescIndOnHKHeaderForIdent ON HkHead (Ident);

/* for table PosVel */
CREATE DESCENDING INDEX DescIndOnPosVelForTIME_UT ON POSVEL (TIME_UT);
CREATE ASCENDING  INDEX AscIndOnPosVelForR_KM     ON POSVEL (R_KM);
CREATE DESCENDING INDEX DescIndOnPosVelForR_KM    ON POSVEL (R_KM);

/* for table Prediction */
CREATE DESCENDING INDEX DescIndOnPredictionForTime_UT ON Prediction (TIME_UT);


/* Create user defined exceptions */

CREATE EXCEPTION OVERLAPPED_TIME_INTERVALS 'Time intervals overlapped!';


/* Create various generators */

/* for generation unique IDENT value for new record in PROGRAM table */
CREATE GENERATOR PROGRAM_IDENT_GEN;
/* for generation unique IDENT value for new record in SessionHead table */
CREATE GENERATOR SessionHead_IDENT_GEN;
/* for generation unique IDENT value for new record in FILEDESC table */
CREATE GENERATOR FILEDESC_IDENT_GEN;
/* for generation unique IDENT value for new record in HkHead table */
CREATE GENERATOR HkHead_IDENT_GEN;
/* for generation unique IDENT value for new record in USERREG table */
CREATE GENERATOR USERREG_IDENT_GEN;
/* for generation unique IDENT value for new record in EXPERT table */
CREATE GENERATOR EXPERT_IDENT_GEN;
/* for generation unique IDENT value for new record in RATECAT table */
CREATE GENERATOR RATECAT_IDENT_GEN;
/* for generation unique IDENT value for new record in TRACE_ID table */
CREATE GENERATOR TRACE_ID_IDENT_GEN;
COMMIT;


/* Create various triggers and procedures */

SET TERM !! ;

CREATE PROCEDURE GetPredictForUT( ut TIMESTAMP)
  RETURNS( DMP FLOAT, DPLA FLOAT, FCE FLOAT, FPE FLOAT, FMP FLOAT, PFMP FLOAT,
           MLT FLOAT, MLAT FLOAT, ILAT FLOAT, L FLOAT, RE FLOAT, XSM FLOAT,
           YSM FLOAT, ZSM FLOAT, XMP FLOAT, YMP FLOAT, ZMP FLOAT,
           LON FLOAT, LAT FLOAT) 
AS
DECLARE VARIABLE t1 TIMESTAMP;
DECLARE VARIABLE t2 TIMESTAMP;
DECLARE VARIABLE DMP1   FLOAT;
DECLARE VARIABLE DPLA1  FLOAT;
DECLARE VARIABLE FCE1   FLOAT;
DECLARE VARIABLE FPE1   FLOAT;
DECLARE VARIABLE FMP1   FLOAT;
DECLARE VARIABLE PFMP1  FLOAT;
DECLARE VARIABLE MLT1   FLOAT;
DECLARE VARIABLE MLAT1  FLOAT;
DECLARE VARIABLE ILAT1  FLOAT;
DECLARE VARIABLE L1     DOUBLE PRECISION;
DECLARE VARIABLE RE1    FLOAT;
DECLARE VARIABLE XSM1   FLOAT;
DECLARE VARIABLE YSM1   FLOAT;
DECLARE VARIABLE ZSM1   FLOAT;
DECLARE VARIABLE XMP1   FLOAT;
DECLARE VARIABLE YMP1   FLOAT;
DECLARE VARIABLE ZMP1   FLOAT;
DECLARE VARIABLE LON1   FLOAT;
DECLARE VARIABLE LAT1   FLOAT;
DECLARE VARIABLE DMP2   FLOAT;
DECLARE VARIABLE DPLA2  FLOAT;
DECLARE VARIABLE FCE2   FLOAT;
DECLARE VARIABLE FPE2   FLOAT;
DECLARE VARIABLE FMP2   FLOAT;
DECLARE VARIABLE PFMP2  FLOAT;
DECLARE VARIABLE MLT2   FLOAT;
DECLARE VARIABLE MLAT2  FLOAT;
DECLARE VARIABLE ILAT2  FLOAT;
DECLARE VARIABLE L2     DOUBLE PRECISION;
DECLARE VARIABLE RE2    FLOAT;
DECLARE VARIABLE XSM2   FLOAT;
DECLARE VARIABLE YSM2   FLOAT;
DECLARE VARIABLE ZSM2   FLOAT;
DECLARE VARIABLE XMP2   FLOAT;
DECLARE VARIABLE YMP2   FLOAT;
DECLARE VARIABLE ZMP2   FLOAT;
DECLARE VARIABLE LON2   FLOAT;
DECLARE VARIABLE LAT2   FLOAT;
DECLARE VARIABLE coeff  FLOAT;
BEGIN
  SELECT MAX(Time_UT) FROM Prediction WHERE Time_UT <= :ut INTO :t1;
  SELECT MIN(Time_UT) FROM Prediction WHERE Time_UT >= :ut INTO :t2;
  IF( t1 <> '1900.01.01' ) THEN
    IF( ut - t1 >= 0.2 ) THEN t1 = '1900.01.01';
  IF( t2 <> '1900.01.01' ) THEN
    IF( t2 - ut >= 0.2 ) THEN t2 = '1900.01.01';
  IF( t1 <> '1900.01.01' ) THEN
  BEGIN
    SELECT DMP, DPLA, FCE, FPE, FMP, PFMP, MLT, MLAT, ILAT, L, RE,
           XSM, YSM, ZSM, XMP, YMP, ZMP, LON, LAT
      FROM Prediction
      WHERE Time_UT = :t1
      INTO :DMP1, :DPLA1, :FCE1, :FPE1, :FMP1, :PFMP1, :MLT1, :MLAT1, :ILAT1,
           :L1, :RE1, :XSM1, :YSM1, :ZSM1, :XMP1, :YMP1, :ZMP1, :LON1, :LAT1;
    IF( t2 <> t1 AND t2 <> '1900.01.01' ) THEN
    BEGIN
      SELECT DMP, DPLA, FCE, FPE, FMP, PFMP, MLT, MLAT, ILAT, L, RE,
             XSM, YSM, ZSM, XMP, YMP, ZMP, LON, LAT
        FROM Prediction
        WHERE Time_UT = :t2
        INTO :DMP2, :DPLA2, :FCE2, :FPE2, :FMP2, :PFMP2, :MLT2, :MLAT2, :ILAT2,
             :L2, :RE2, :XSM2, :YSM2, :ZSM2, :XMP2, :YMP2, :ZMP2, :LON2, :LAT2;
      coeff = ( ut - t1 ) / ( t2 - t1 );
      DMP = DMP1 + ( DMP2 - DMP1 ) * coeff;
      DPLA = DPLA1 + ( DPLA2 - DPLA1 ) * coeff;
      FCE  = FCE1 + ( FCE2 - FCE1 ) * coeff;
      FPE  = FPE1 + ( FPE2 - FPE1 ) * coeff;
      FMP  = FMP1 + ( FMP2 - FMP1 ) * coeff;
      PFMP = PFMP1 + ( PFMP2 - PFMP1 ) * coeff;
      MLT  = MLT1 + ( MLT2 - MLT1 ) * coeff;
      MLAT = MLAT1 + ( MLAT2 - MLAT1 ) * coeff;
      ILAT = ILAT1 + ( ILAT2 - ILAT1 ) * coeff;
      L    = L1 + ( L2 - L1 ) * coeff;
      RE   = RE1 + ( RE2 - RE1 ) * coeff;
      XSM  = XSM1 + ( XSM2 - XSM1 ) * coeff;
      YSM  = YSM1 + ( YSM2 - YSM1 ) * coeff;
      ZSM  = ZSM1 + ( ZSM2 - ZSM1 ) * coeff;
      XMP  = XMP1 + ( XMP2 - XMP1 ) * coeff;
      YMP  = YMP1 + ( YMP2 - YMP1 ) * coeff;
      ZMP  = ZMP1 + ( ZMP2 - ZMP1 ) * coeff;
      LON  = LON1 + ( LON2 - LON1 ) * coeff;
      LAT  = LAT1 + ( LAT2 - LAT1 ) * coeff;
      SUSPEND;
    END
    ELSE
    BEGIN
      DMP  = DMP1;
      DPLA = DPLA1;
      FCE  = FCE1;
      FPE  = FPE1;
      FMP  = FMP1;
      PFMP = PFMP1;
      MLT  = MLT1;
      MLAT = MLAT1;
      ILAT = ILAT1;
      L    = L1;
      RE   = RE1;
      XSM  = XSM1;
      YSM  = YSM1;
      ZSM  = ZSM1;
      XMP  = XMP1;
      YMP  = YMP1;
      ZMP  = ZMP1;
      LON  = LON1;
      LAT  = LAT1;
      SUSPEND;
    END
  END
  ELSE
  BEGIN
    IF( t2 <> '1900.01.01' ) then
    BEGIN
      SELECT DMP, DPLA, FCE, FPE, FMP, PFMP, MLT, MLAT, ILAT, L, RE,
             XSM, YSM, ZSM, XMP, YMP, ZMP, LON, LAT
        FROM Prediction
        WHERE Time_UT = :t2
        INTO :DMP, :DPLA, :FCE, :FPE, :FMP, :PFMP, :MLT, :MLAT, :ILAT,
             :L, :RE, :XSM, :YSM, :ZSM, :XMP, :YMP, :ZMP, :LON, :LAT;
      SUSPEND;
    END
  END
END!!

/* Insert trigger for PROGRAM table to automatically generate unique IDENT value */
CREATE TRIGGER CREATE_PROGRAM_IDENT FOR PROGRAM
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(PROGRAM_IDENT_GEN, 1); 
	END!!

/* Insert trigger for SessionHead table to automatically generate
   unique IDENT value */
CREATE TRIGGER CREATE_SessionHead_IDENT FOR SessionHead
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(SessionHead_IDENT_GEN, 1);
	END!!

/* Insert trigger for FILEDESC table to automatically generate unique IDENT value */
CREATE TRIGGER CREATE_FILEDESC_IDENT FOR FILEDESC
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(FILEDESC_IDENT_GEN, 1); 
	END!!

/* Insert trigger for HkHead table to automatically generate unique IDENT value */
CREATE TRIGGER CREATE_HkHead_IDENT FOR HkHead
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(HkHead_IDENT_GEN, 1);
	END!!

/* Insert trigger for USERREG table to automatically generate unique IDENT value */
CREATE TRIGGER CREATE_USERREG_IDENT FOR USERREG
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(USERREG_IDENT_GEN, 1); 
	END!!

/* Insert trigger for EXPERT table to automatically generate unique IDENT value */
CREATE TRIGGER CREATE_EXPERT_IDENT FOR EXPERT
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(EXPERT_IDENT_GEN, 1); 
	END!!

/* Insert trigger for RATECAT table to automatically generate unique IDENT value */
CREATE TRIGGER CREATE_RATECAT_IDENT FOR RATECAT
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(RATECAT_IDENT_GEN, 1); 
	END!!

/* Insert trigger for TRACE_ID table to automatically generate unique IDENT value */
CREATE TRIGGER CREATE_TRACE_ID_IDENT FOR TRACE_ID
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.IDENT = GEN_ID(TRACE_ID_IDENT_GEN, 1); 
	END!!

/* Insert trigger for DATABIN table to automatically convert NAME field to upper case */
CREATE TRIGGER INS_CONVERT_DATABIN_UPPER FOR DATABIN
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.NAME = UPPER( NEW.NAME ); 
	END!!

/* Update trigger for DATABIN table to automatically convert NAME field to upper case */
CREATE TRIGGER UPD_CONVERT_DATABIN_UPPER FOR DATABIN
	BEFORE UPDATE POSITION 0
	AS BEGIN
		NEW.NAME = UPPER( NEW.NAME ); 
	END!!

/* Insert trigger for OPMODE table to automatically convert NAME field to upper case */
CREATE TRIGGER INS_CONVERT_OPMODE_UPPER FOR OPMODE
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.NAME = UPPER( NEW.NAME ); 
	END!!

/* Update trigger for OPMODE table to automatically convert NAME field to upper case */
CREATE TRIGGER UPD_CONVERT_OPMODE_UPPER FOR OPMODE
	BEFORE UPDATE POSITION 0
	AS BEGIN
		NEW.NAME = UPPER( NEW.NAME ); 
	END!!

/* Insert trigger for WAVEFORM table to automatically convert NAME field to upper case */
CREATE TRIGGER INS_CONVERT_WAVEFORM_UPPER FOR WAVEFORM
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.NAME = UPPER( NEW.NAME ); 
	END!!

/* Update trigger for WAVEFORM table to automatically convert NAME field to upper case */
CREATE TRIGGER UPD_CONVERT_WAVEFORM_UPPER FOR WAVEFORM
	BEFORE UPDATE POSITION 0
	AS BEGIN
		NEW.NAME = UPPER( NEW.NAME ); 
	END!!

/* Insert trigger for EXPERT table to automatically convert LOGNAME field to upper case */
CREATE TRIGGER INS_CONVERT_EXPERT_UPPER FOR EXPERT
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.LOGNAME = UPPER( NEW.LOGNAME ); 
	END!!

/* Update trigger for EXPERT table to automatically convert LOGNAME field to upper case */
CREATE TRIGGER UPD_CONVERT_EXPERT_UPPER FOR EXPERT
	BEFORE UPDATE POSITION 0
	AS BEGIN
		NEW.LOGNAME = UPPER( NEW.LOGNAME ); 
	END!!

/* Insert trigger for RATECAT table to automatically convert SHORTNAME field to upper case */
CREATE TRIGGER INS_CONVERT_RATECAT_UPPER FOR RATECAT
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.SHORTNAME = UPPER( NEW.SHORTNAME ); 
	END!!

/* Update trigger for RATECAT table to automatically convert SHORTNAME field to upper case */
CREATE TRIGGER UPD_CONVERT_RATECAT_UPPER FOR RATECAT
	BEFORE UPDATE POSITION 0
	AS BEGIN
		NEW.SHORTNAME = UPPER( NEW.SHORTNAME ); 
	END!!

/* Insert trigger for TRPARCAT table to automatically convert SHORTNAME field to upper case */
CREATE TRIGGER INS_CONVERT_TRPARCAT_UPPER FOR TRPARCAT
	BEFORE INSERT POSITION 0
	AS BEGIN
		NEW.SHORTNAME = UPPER( NEW.SHORTNAME ); 
	END!!

/* Update trigger for TRPARCAT_ID table to automatically convert SHORTNAME field to upper case */
CREATE TRIGGER UPD_CONVERT_TRPARCAT_UPPER FOR TRPARCAT
	BEFORE UPDATE POSITION 0
	AS BEGIN
		NEW.SHORTNAME = UPPER( NEW.SHORTNAME ); 
	END!!

/* Update trigger for UserReg table to automatically increase COUNT field value */
CREATE TRIGGER UPD_USERREG_INC_COUNT FOR USERREG
	BEFORE UPDATE POSITION 0
	AS BEGIN
		NEW.COUNT_ACT = OLD.COUNT_ACT + 1; 
		NEW.SRV_LOCAL_TIME = CURRENT_TIMESTAMP;
	END!!

/* Update trigger for AllRates table to automatically update SubmissionUT field */
CREATE TRIGGER UPD_AllRates_Submission FOR AllRates
	BEFORE UPDATE POSITION 0
	AS BEGIN
          IF( NEW.Rate <> OLD.Rate AND NEW.SubmissionUT = OLD.SubmissionUT ) THEN
            NEW.SubmissionUT = CURRENT_TIMESTAMP;
	END!!

/* Update trigger for AllSnComm table to automatically update SubmissionUT field */
CREATE TRIGGER UPD_AllSnComm_Submission FOR AllSnComm
	BEFORE UPDATE POSITION 0
	AS BEGIN
          IF( NEW.SubmissionUT = OLD.SubmissionUT ) THEN
            NEW.SubmissionUT = CURRENT_TIMESTAMP;
	END!!

/* Update trigger for TracePar table to automatically update SubmissionUT field */
CREATE TRIGGER UPD_TracePar_Submission FOR TracePar
	BEFORE UPDATE POSITION 0
	AS BEGIN
          IF( NEW.Param <> OLD.Param AND NEW.SubmissionUT = OLD.SubmissionUT ) THEN
            NEW.SubmissionUT = CURRENT_TIMESTAMP;
	END!!

SET TERM ; !!
COMMIT;

/* Create Roles */
CREATE ROLE Expert;
CREATE ROLE Common;
GRANT Common TO BinBrowser;
COMMIT;

/* Create Grants */

/* !!!don't delete this grant - it's for connection availability checking!!! */
GRANT SELECT, INSERT, UPDATE ON USERREG TO UCONNECT;

GRANT SELECT ON DATABIN     TO Common, Expert;
GRANT SELECT ON HKTYPE      TO Common, Expert;
GRANT SELECT ON OPMODE      TO Common, Expert;
GRANT SELECT ON WAVEFORM    TO Common, Expert;
GRANT SELECT ON TXANTENNA   TO Common, Expert;
GRANT SELECT ON FILEDESC    TO Common, Expert;
GRANT SELECT ON PROGRAM     TO Common, Expert;
GRANT SELECT ON CLOCKDRIFT  TO Common, Expert;
GRANT SELECT ON SessionHead TO Common, Expert;
GRANT SELECT ON SessionBlob TO Common, Expert;
GRANT SELECT ON Session     TO Common, Expert;
GRANT SELECT ON HkHead      TO Common, Expert;
GRANT SELECT ON HkBlob      TO Common, Expert;
GRANT SELECT ON HkHeader    TO Common, Expert;
GRANT SELECT ON AuxPar      TO Common, Expert;
GRANT SELECT ON ORBITPAR    TO Common, Expert;
GRANT SELECT ON PASS        TO Common, Expert;
GRANT SELECT ON POSVEL      TO Common, Expert;
GRANT SELECT ON SPREDICT    TO Common, Expert;
GRANT SELECT ON HPREDICT    TO Common, Expert;
GRANT SELECT ON EXPERT      TO Common, Expert;
GRANT SELECT ON RATECAT     TO Common, Expert;
GRANT SELECT ON RATE        TO Common, Expert;
GRANT SELECT ON SNCOMMENT   TO Common, Expert;
GRANT SELECT ON SNBBVPAR    TO Common, Expert;
GRANT SELECT ON TRACEPAR    TO Common, Expert;
GRANT SELECT ON TRACE_ID    TO Common, Expert;
GRANT SELECT ON TRCOMMENT   TO Common, Expert;
GRANT SELECT ON TRPARCAT    TO Common, Expert;
GRANT SELECT ON TRACECAT    TO Common, Expert;
GRANT SELECT ON TRACEDATA   TO Common, Expert;
GRANT SELECT ON RESPARCAT   TO Common, Expert;
GRANT SELECT ON RESCAT      TO Common, Expert;
GRANT SELECT ON RESPAR      TO Common, Expert;
GRANT SELECT ON RESDATA     TO Common, Expert;
GRANT SELECT ON Prediction  TO Common, Expert;
GRANT SELECT ON VERSION     TO Common, Expert;
GRANT INSERT, UPDATE, DELETE ON RATE      TO EXPERT;
GRANT SELECT, INSERT, UPDATE, DELETE ON ALLRATES  TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON SNCOMMENT TO EXPERT;
GRANT SELECT, INSERT, UPDATE, DELETE ON ALLSNCOMM TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON TRACE_ID  TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON TRCOMMENT TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON TRACEPAR  TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON TRACEDATA TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON RESPAR TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON RESDATA TO EXPERT;
GRANT INSERT, UPDATE, DELETE ON SNBBVPAR  TO EXPERT;
GRANT EXECUTE ON PROCEDURE GetPredictForUT TO Common, Expert;
COMMIT;
